import React, { useState, useEffect } from 'react'
import { withClient, Client } from 'cozy-client'

import { Konnector, Trigger } from 'doctypes'
import KonnectorService from 'services/konnectorService'
import KonnectorStatusService from 'services/konnectorStatusService'

import KonnectorViewerCard from 'components/ContentComponents/KonnectorViewer/KonnectorViewerCard'
import IFluidConfig from 'services/IFluidConfig'

export interface KonnectorViewerProps {
  fluidConfig: IFluidConfig
  client: Client
  isParam: boolean
}

const KonnectorViewer: React.FC<KonnectorViewerProps> = ({
  fluidConfig,
  client,
  isParam = false,
}: KonnectorViewerProps) => {
  const [konnector, setKonnector] = useState<Konnector | null>(null)
  const [lastTrigger, setLastTrigger] = useState<Trigger | null>(null)
  const [loaded, setLoaded] = useState<boolean>(false)

  useEffect(() => {
    let subscribed = true
    const konnectorService = new KonnectorService(
      client,
      fluidConfig.konnectorConfig.slug
    )
    const kss = new KonnectorStatusService(client)

    async function getData() {
      const _konnector: Konnector = await konnectorService.fetchKonnector()
      if (subscribed && _konnector) {
        setKonnector(_konnector)
        const allTriggers = await kss.getAllTriggers()
        if (allTriggers) {
          const lastTrigger = allTriggers
            .filter(
              (trigger: Trigger) =>
                trigger.worker === 'konnector' &&
                trigger.message.konnector === fluidConfig.konnectorConfig.slug
            )
            .sort((a: string | number | Date, b: string | number | Date) =>
              new Date(a) > new Date(b) ? 1 : -1
            )
            .shift()
          if (subscribed && lastTrigger) {
            setLastTrigger(lastTrigger)
          }
        }
      }
      if (subscribed) {
        setLoaded(true)
      }
    }
    getData()
    return () => {
      subscribed = false
    }
  }, [])

  return (
    <>
      {!loaded ? null : (
        <KonnectorViewerCard
          fluidConfig={fluidConfig}
          konnector={konnector}
          trigger={lastTrigger}
          isParam={isParam}
        />
      )}
    </>
  )
}

export default withClient(KonnectorViewer)
